@@ -3,7 +3,7 @@
 # This file is part of Radicale Server - Calendar Server
 # Copyright © 2008 Nicolas Kandel
 # Copyright © 2008 Pascal Halter
-# Copyright © 2008-2013 Guillaume Ayoub
+# Copyright © 2008-2015 Guillaume Ayoub
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,10 +26,14 @@ Define the main classes of a collection as seen from the server.
 """
 
 import os
-import posixpath
-import uuid
+import hashlib
+import re
+from uuid import uuid4
+from random import randint
 from contextlib import contextmanager
 
+from . import pathutils
+
 
 def serialize(tag, headers=(), items=()):
     """Return a text corresponding to given collection ``tag``.
@@ -38,6 +42,7 @@ def serialize(tag, headers=(), items=()):
     items if needed (ie. for calendars).
 
     """
+    items = sorted(items, key=lambda x: x.name)
     if tag == "VADDRESSBOOK":
         lines = [item.text for item in items]
     else:
@@ -55,13 +60,7 @@ def unfold(text):
     Read rfc5545-3.1 for info.
 
     """
-    lines = []
-    for line in text.splitlines():
-        if lines and (line.startswith(" ") or line.startswith("\t")):
-            lines[-1] += line[1:]
-        else:
-            lines.append(line)
-    return lines
+    return re.sub('\r\n( |\t)', '', text).splitlines()
 
 
 class Item(object):
@@ -102,10 +101,17 @@ class Item(object):
                 self.text = self.text.replace(
                     "\nEND:", "\nX-RADICALE-NAME:%s\nEND:" % self._name)
         else:
-            self._name = str(uuid.uuid4())
+            # workaround to get unicode on both python2 and 3
+            self._name = uuid4().hex.encode("ascii").decode("ascii")
             self.text = self.text.replace(
                 "\nEND:", "\nX-RADICALE-NAME:%s\nEND:" % self._name)
 
+    def __hash__(self):
+        return hash(self.text)
+
+    def __eq__(self, item):
+        return isinstance(item, Item) and self.text == item.text
+
     @property
     def etag(self):
         """Item etag.
@@ -113,7 +119,9 @@ class Item(object):
         Etag is mainly used to know if an item has changed.
 
         """
-        return '"%s"' % hash(self.text)
+        md5 = hashlib.md5()
+        md5.update(self.text.encode("utf-8"))
+        return '"%s"' % md5.hexdigest()
 
     @property
     def name(self):
@@ -176,8 +184,9 @@ class Collection(object):
 
         """
         self.encoding = "utf-8"
-        split_path = path.split("/")
-        self.path = path if path != "." else ""
+        # path should already be sanitized
+        self.path = pathutils.sanitize_path(path).strip("/")
+        split_path = self.path.split("/")
         if principal and split_path and self.is_node(self.path):
             # Already existing principal collection
             self.owner = split_path[0]
@@ -187,6 +196,7 @@ class Collection(object):
         else:
             self.owner = None
         self.is_principal = principal
+        self._items = None
 
     @classmethod
     def from_path(cls, path, depth="1", include_container=True):
@@ -207,12 +217,15 @@ class Collection(object):
         if path is None:
             return []
 
-        # First do normpath and then strip, to prevent access to FOLDER/../
-        sane_path = posixpath.normpath(path.replace(os.sep, "/")).strip("/")
+        # path should already be sanitized
+        sane_path = pathutils.sanitize_path(path).strip("/")
         attributes = sane_path.split("/")
         if not attributes:
             return []
-        if not (cls.is_leaf("/".join(attributes)) or path.endswith("/")):
+
+        # Try to guess if the path leads to a collection or an item
+        if (cls.is_leaf("/".join(attributes[:-1])) or not
+                path.endswith(("/", "/caldav", "/carddav"))):
             attributes.pop()
 
         result = []
@@ -299,7 +312,7 @@ class Collection(object):
 
         If ``name`` is given, give this name to new items in ``text``.
 
-        Return a list of items.
+        Return a dict of items.
 
         """
         item_tags = {}
@@ -332,13 +345,7 @@ class Collection(object):
                     else:
                         items[item.name] = item
 
-        return list(items.values())
-
-    def get_item(self, name):
-        """Get collection item called ``name``."""
-        for item in self.items:
-            if item.name == name:
-                return item
+        return items
 
     def append(self, name, text):
         """Append items from ``text`` to collection.
@@ -346,37 +353,27 @@ class Collection(object):
         If ``name`` is given, give this name to new items in ``text``.
 
         """
-        items = self.items
-
-        for new_item in self._parse(
-                text, (Timezone, Event, Todo, Journal, Card), name):
-            if new_item.name not in (item.name for item in items):
-                items.append(new_item)
-
-        self.write(items=items)
+        new_items = self._parse(
+            text, (Timezone, Event, Todo, Journal, Card), name)
+        for new_item in new_items.values():
+            if new_item.name not in self.items:
+                self.items[new_item.name] = new_item
+        self.write()
 
     def remove(self, name):
         """Remove object named ``name`` from collection."""
-        components = [
-            component for component in self.components
-            if component.name != name]
-
-        items = self.timezones + components
-        self.write(items=items)
+        if name in self.items:
+            del self.items[name]
+        self.write()
 
     def replace(self, name, text):
         """Replace content by ``text`` in collection objet called ``name``."""
         self.remove(name)
         self.append(name, text)
 
-    def write(self, headers=None, items=None):
+    def write(self):
         """Write collection with given parameters."""
-        headers = headers or self.headers or (
-            Header("PRODID:-//Radicale//NONSGML Radicale Server//EN"),
-            Header("VERSION:%s" % self.version))
-        items = items if items is not None else self.items
-
-        text = serialize(self.tag, headers, items)
+        text = serialize(self.tag, self.headers, self.items.values())
         self.save(text)
 
     def set_mimetype(self, mimetype):
@@ -396,7 +393,7 @@ class Collection(object):
                 try:
                     tag = open(self.path).readlines()[0][6:].rstrip()
                 except IOError:
-                    if self.path.endswith(".vcf"):
+                    if self.path.endswith((".vcf", "/carddav")):
                         props["tag"] = "VADDRESSBOOK"
                     else:
                         props["tag"] = "VCALENDAR"
@@ -426,7 +423,9 @@ class Collection(object):
     @property
     def etag(self):
         """Etag from collection."""
-        return '"%s"' % hash(self.text)
+        md5 = hashlib.md5()
+        md5.update(self.text.encode("utf-8"))
+        return '"%s"' % md5.hexdigest()
 
     @property
     def name(self):
@@ -434,62 +433,53 @@ class Collection(object):
         with self.props as props:
             return props.get("D:displayname", self.path.split(os.path.sep)[-1])
 
+    @property
+    def color(self):
+        """Collection color."""
+        with self.props as props:
+            if "ICAL:calendar-color" not in props:
+                props["ICAL:calendar-color"] = "#%x" % randint(0, 255 ** 3 - 1)
+            return props["ICAL:calendar-color"]
+
     @property
     def headers(self):
         """Find headers items in collection."""
         header_lines = []
 
-        lines = unfold(self.text)
-        for header in ("PRODID", "VERSION"):
-            for line in lines:
-                if line.startswith("%s:" % header):
-                    header_lines.append(Header(line))
-                    break
+        lines = unfold(self.text)[1:]
+        for line in lines:
+            if line.startswith(("BEGIN:", "END:")):
+                break
+            header_lines.append(Header(line))
 
-        return header_lines
+        return header_lines or (
+            Header("PRODID:-//Radicale//NONSGML Radicale Server//EN"),
+            Header("VERSION:%s" % self.version))
 
     @property
     def items(self):
         """Get list of all items in collection."""
-        return self._parse(self.text, (Event, Todo, Journal, Card, Timezone))
+        if self._items is None:
+            self._items = self._parse(
+                self.text, (Event, Todo, Journal, Card, Timezone))
+        return self._items
+
+    @property
+    def timezones(self):
+        """Get list of all timezones in collection."""
+        return [
+            item for item in self.items.values() if item.tag == Timezone.tag]
 
     @property
     def components(self):
         """Get list of all components in collection."""
-        return self._parse(self.text, (Event, Todo, Journal, Card))
-
-    @property
-    def events(self):
-        """Get list of ``Event`` items in calendar."""
-        return self._parse(self.text, (Event,))
-
-    @property
-    def todos(self):
-        """Get list of ``Todo`` items in calendar."""
-        return self._parse(self.text, (Todo,))
-
-    @property
-    def journals(self):
-        """Get list of ``Journal`` items in calendar."""
-        return self._parse(self.text, (Journal,))
-
-    @property
-    def timezones(self):
-        """Get list of ``Timezome`` items in calendar."""
-        return self._parse(self.text, (Timezone,))
-
-    @property
-    def cards(self):
-        """Get list of ``Card`` items in address book."""
-        return self._parse(self.text, (Card,))
+        tags = [item_type.tag for item_type in (Event, Todo, Journal, Card)]
+        return [item for item in self.items.values() if item.tag in tags]
 
     @property
     def owner_url(self):
         """Get the collection URL according to its owner."""
-        if self.owner:
-            return "/%s/" % self.owner
-        else:
-            return None
+        return "/%s/" % self.owner if self.owner else None
 
     @property
     def url(self):
