@@ -3,7 +3,7 @@
 # This file is part of Radicale Server - Calendar Server
 # Copyright © 2008 Nicolas Kandel
 # Copyright © 2008 Pascal Halter
-# Copyright © 2008-2013 Guillaume Ayoub
+# Copyright © 2008-2015 Guillaume Ayoub
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -49,7 +49,6 @@ from . import client, config, ical
 
 
 NAMESPACES = {
-    "A": "http://apple.com/ns/ical/",
     "C": "urn:ietf:params:xml:ns:caldav",
     "CR": "urn:ietf:params:xml:ns:carddav",
     "D": "DAV:",
@@ -228,7 +227,7 @@ def propfind(path, xml_request, collections, user=None):
                  _tag("D", "displayname"),
                  _tag("D", "owner"),
                  _tag("D", "getetag"),
-                 _tag("A", "calendar-color"),
+                 _tag("ICAL", "calendar-color"),
                  _tag("CS", "getctag")]
 
     # Writing answer
@@ -297,10 +296,7 @@ def _propfind_response(path, item, props, user):
             # pylint: enable=W0511
         elif tag == _tag("D", "current-user-principal") and user:
             tag = ET.Element(_tag("D", "href"))
-            if item.resource_type == "addressbook":
-                tag.text = _href("/%s/addressbook.vcf/" % user)
-            else:
-                tag.text = _href("/%s/calendar.ics/" % user)
+            tag.text = _href("/%s/" % user)
             element.append(tag)
         elif tag == _tag("D", "current-user-privilege-set"):
             privilege = ET.Element(_tag("D", "privilege"))
@@ -346,7 +342,7 @@ def _propfind_response(path, item, props, user):
                     item.tag, item.headers, item.timezones)
             elif tag == _tag("D", "displayname"):
                 element.text = item.name
-            elif tag == _tag("A", "calendar-color"):
+            elif tag == _tag("ICAL", "calendar-color"):
                 element.text = item.color
             else:
                 human_tag = _tag_from_clark(tag)
@@ -449,12 +445,15 @@ def proppatch(path, xml_request, collection):
 def put(path, ical_request, collection):
     """Read PUT requests."""
     name = name_from_path(path, collection)
-    if name in (item.name for item in collection.items):
+    if name in collection.items:
         # PUT is modifying an existing item
         collection.replace(name, ical_request)
-    else:
+    elif name:
         # PUT is adding a new item
         collection.append(name, ical_request)
+    else:
+        # PUT is replacing the whole collection
+        collection.save(ical_request)
 
 
 def report(path, xml_request, collection):
@@ -467,7 +466,9 @@ def report(path, xml_request, collection):
     root = ET.fromstring(xml_request.encode("utf8"))
 
     prop_element = root.find(_tag("D", "prop"))
-    props = [prop.tag for prop in prop_element]
+    props = (
+        [prop.tag for prop in prop_element]
+        if prop_element is not None else [])
 
     if collection:
         if root.tag in (_tag("C", "calendar-multiget"),
@@ -496,7 +497,6 @@ def report(path, xml_request, collection):
     multistatus = ET.Element(_tag("D", "multistatus"))
 
     collection_tag = collection.tag
-    collection_items = collection.items
     collection_headers = collection.headers
     collection_timezones = collection.timezones
 
@@ -506,46 +506,84 @@ def report(path, xml_request, collection):
         if name:
             # Reference is an item
             path = "/".join(hreference.split("/")[:-1]) + "/"
-            items = (item for item in collection_items if item.name == name)
+            try:
+                items = [collection.items[name]]
+            except KeyError:
+                multistatus.append(
+                    _item_response(hreference, found_item=False))
+                continue
+
         else:
             # Reference is a collection
             path = hreference
             items = collection.components
 
         for item in items:
+            href = _href("%s/%s" % (path.rstrip("/"), item.name))
             if tag_filters and item.tag not in tag_filters:
                 continue
 
-            response = ET.Element(_tag("D", "response"))
-            multistatus.append(response)
-
-            href = ET.Element(_tag("D", "href"))
-            href.text = _href("%s/%s" % (path.rstrip("/"), item.name))
-            response.append(href)
-
-            propstat = ET.Element(_tag("D", "propstat"))
-            response.append(propstat)
-
-            prop = ET.Element(_tag("D", "prop"))
-            propstat.append(prop)
+            found_props = []
+            not_found_props = []
 
             for tag in props:
                 element = ET.Element(tag)
                 if tag == _tag("D", "getetag"):
                     element.text = item.etag
+                    found_props.append(element)
                 elif tag == _tag("D", "getcontenttype"):
                     element.text = "%s; component=%s" % (
                         item.mimetype, item.tag.lower())
+                    found_props.append(element)
                 elif tag in (_tag("C", "calendar-data"),
                              _tag("CR", "address-data")):
                     if isinstance(item, ical.Component):
                         element.text = ical.serialize(
                             collection_tag, collection_headers,
                             collection_timezones + [item])
-                prop.append(element)
+                    found_props.append(element)
+                else:
+                    not_found_props.append(element)
 
-            status = ET.Element(_tag("D", "status"))
-            status.text = _response(200)
-            propstat.append(status)
+            multistatus.append(_item_response(
+                href, found_props=found_props, not_found_props=not_found_props,
+                found_item=True))
 
     return _pretty_xml(multistatus)
+
+
+def _item_response(href, found_props=(), not_found_props=(), found_item=True):
+    response = ET.Element(_tag("D", "response"))
+
+    href_tag = ET.Element(_tag("D", "href"))
+    href_tag.text = href
+    response.append(href_tag)
+
+    if found_item:
+        if found_props:
+            propstat = ET.Element(_tag("D", "propstat"))
+            status = ET.Element(_tag("D", "status"))
+            status.text = _response(200)
+            prop = ET.Element(_tag("D", "prop"))
+            for p in found_props:
+                prop.append(p)
+            propstat.append(prop)
+            propstat.append(status)
+            response.append(propstat)
+
+        if not_found_props:
+            propstat = ET.Element(_tag("D", "propstat"))
+            status = ET.Element(_tag("D", "status"))
+            status.text = _response(404)
+            prop = ET.Element(_tag("D", "prop"))
+            for p in not_found_props:
+                prop.append(p)
+            propstat.append(prop)
+            propstat.append(status)
+            response.append(propstat)
+    else:
+        status = ET.Element(_tag("D", "status"))
+        status.text = _response(404)
+        response.append(status)
+
+    return response
